
# This is the Perl OpenGL build configuration file.
# It contains the final OpenGL build arguments from
# the configuration process.  Access the values by
# use OpenGL::Config which defines the variable
# $OpenGL::Config containing the hash arguments from
# the WriteMakefile() call.
#
$OpenGL::Config = {
                    'DEFINE' => '-DHAVE_VER -DHAVE_FREEGLUT -DHAVE_GL -DHAVE_GLU -DHAVE_GLX -DHAVE_FREEGLUT -DHAVE_FREEGLUT_H -DGL_GLEXT_LEGACY',
                    'INC' => '',
                    'LIBS' => '  -lGL -lgl -lglut -lX11 -lglu -lm'
                  };

1;
__END__
